@@ -2,17 +2,17 @@
 # 2.0, and the MIT License.  See the LICENSE file in the root of this
 # repository for complete details.
 
-from __future__ import absolute_import, division, print_function
 
 import pytest
 
 from pretend import raiser, stub
 
+from structlog import get_context
 from structlog._base import BoundLoggerBase
 from structlog._config import _CONFIG
-from structlog._loggers import ReturnLogger
 from structlog.exceptions import DropEvent
 from structlog.processors import KeyValueRenderer
+from structlog.testing import ReturnLogger
 
 
 def build_bl(logger=None, processors=None, context=None):
@@ -26,7 +26,7 @@ def build_bl(logger=None, processors=None, context=None):
     )
 
 
-class TestBinding(object):
+class TestBinding:
     def test_repr(self):
         bl = build_bl(processors=[1, 2, 3], context={})
 
@@ -46,20 +46,26 @@ class TestBinding(object):
         assert b._context != b1._context != b2._context
 
     def test_new_clears_state(self):
+        """
+        Calling new() on a logger clears the context.
+        """
         b = build_bl()
         b = b.bind(x=42)
 
-        assert 42 == b._context["x"]
+        assert 42 == get_context(b)["x"]
 
         b = b.bind()
 
-        assert 42 == b._context["x"]
+        assert 42 == get_context(b)["x"]
 
         b = b.new()
 
-        assert "x" not in b._context
+        assert {} == dict(get_context(b))
 
     def test_comparison(self):
+        """
+        Two bound loggers are equal if their context is equal.
+        """
         b = build_bl()
 
         assert b == b.bind()
@@ -68,6 +74,10 @@ class TestBinding(object):
         assert b != "test"
 
     def test_bind_keeps_class(self):
+        """
+        Binding values does not change the type of the bound logger.
+        """
+
         class Wrapper(BoundLoggerBase):
             pass
 
@@ -76,6 +86,10 @@ class TestBinding(object):
         assert isinstance(b.bind(), Wrapper)
 
     def test_new_keeps_class(self):
+        """
+        Clearing context does not change the type of the bound logger.
+        """
+
         class Wrapper(BoundLoggerBase):
             pass
 
@@ -115,7 +129,7 @@ class TestBinding(object):
         assert {"y": 23} == b._context
 
 
-class TestProcessing(object):
+class TestProcessing:
     def test_event_empty_string(self):
         """
         Empty strings are a valid event.
@@ -143,7 +157,12 @@ class TestProcessing(object):
         assert "event" not in b._context
 
     def test_chain_does_not_swallow_all_exceptions(self):
+        """
+        If the chain raises anything else than DropEvent, the error is not
+        swallowed.
+        """
         b = build_bl(processors=[raiser(ValueError)])
+
         with pytest.raises(ValueError):
             b._process_event("", "boom", {})
 
@@ -157,6 +176,16 @@ class TestProcessing(object):
 
         assert (("foo",), {}) == b._process_event("", "foo", {})
 
+    def test_last_processor_returns_bytes(self):
+        """
+        If the final processor returns bytes, ``(the_bytes,), {}`` is
+        returned.
+        """
+        logger = stub(msg=lambda *args, **kw: (args, kw))
+        b = build_bl(logger, processors=[lambda *_: b"foo"])
+
+        assert ((b"foo",), {}) == b._process_event(None, "name", {})
+
     def test_last_processor_returns_tuple(self):
         """
         If the final processor returns a tuple, it is just passed through.
@@ -190,7 +219,7 @@ class TestProcessing(object):
         assert exc.value.args[0].startswith("Last processor didn't return")
 
 
-class TestProxying(object):
+class TestProxying:
     def test_processor_raising_DropEvent_silently_aborts_chain(self, capsys):
         """
         If a processor raises DropEvent, the chain is aborted and nothing is
