@@ -1,18 +1,15 @@
-# -*- coding: utf-8 -*-
-
 # This file is dual licensed under the terms of the Apache License, Version
 # 2.0, and the MIT License.  See the LICENSE file in the root of this
 # repository for complete details.
 
-from __future__ import absolute_import, division, print_function
+import pickle
 
 import pytest
-import six
 
 from structlog import dev
 
 
-class TestPad(object):
+class TestPad:
     def test_normal(self):
         """
         If chars are missing, adequate number of " " are added.
@@ -48,7 +45,7 @@ def unpadded(styles):
     return styles.bright + "test" + styles.reset
 
 
-class TestConsoleRenderer(object):
+class TestConsoleRenderer:
     @pytest.mark.skipif(dev._has_colorama, reason="Colorama must be missing.")
     def test_missing_colorama(self):
         """
@@ -56,7 +53,7 @@ class TestConsoleRenderer(object):
         colorama is missing.
         """
         with pytest.raises(SystemError) as e:
-            dev.ConsoleRenderer()
+            dev.ConsoleRenderer(colors=True)
 
         assert (
             "ConsoleRenderer with `colors=True` requires the colorama package "
@@ -79,6 +76,16 @@ class TestConsoleRenderer(object):
 
         assert (styles.timestamp + "42" + styles.reset + " " + unpadded) == rv
 
+    def test_event_stringified(self, cr, styles, unpadded):
+        """
+        Event is cast to string.
+        """
+        not_a_string = Exception("test")
+
+        rv = cr(None, None, {"event": not_a_string})
+
+        assert unpadded == rv
+
     def test_level(self, cr, styles, padded):
         """
         Levels are rendered aligned, in square brackets, and color coded.
@@ -288,7 +295,44 @@ class TestConsoleRenderer(object):
         )
 
         cnt = rv.count("哈")
-        if rns and six.PY2:
-            assert 1 == cnt
-        else:
-            assert 2 == cnt
+
+        assert 2 == cnt
+
+    @pytest.mark.parametrize("repr_native_str", [True, False])
+    @pytest.mark.parametrize("force_colors", [True, False])
+    @pytest.mark.parametrize("proto", range(pickle.HIGHEST_PROTOCOL))
+    def test_pickle(self, repr_native_str, force_colors, proto):
+        """
+        ConsoleRenderer can be pickled and unpickled.
+        """
+        r = dev.ConsoleRenderer(
+            repr_native_str=repr_native_str, force_colors=force_colors
+        )
+
+        assert r(None, None, {"event": "foo"}) == pickle.loads(
+            pickle.dumps(r, proto)
+        )(None, None, {"event": "foo"})
+
+
+class TestSetExcInfo:
+    def test_wrong_name(self):
+        """
+        Do nothing if name is not exception.
+        """
+        assert {} == dev.set_exc_info(None, "foo", {})
+
+    @pytest.mark.parametrize("ei", [False, None, ()])
+    def test_already_set(self, ei):
+        """
+        Do nothing if exc_info is already set.
+        """
+        assert {"exc_info": ei} == dev.set_exc_info(
+            None, "foo", {"exc_info": ei}
+        )
+
+    def test_set_it(self):
+        """
+        Set exc_info to True if its not set and if the method name is
+        exception.
+        """
+        assert {"exc_info": True} == dev.set_exc_info(None, "exception", {})
