@@ -2,14 +2,17 @@
 # 2.0, and the MIT License.  See the LICENSE file in the root of this
 # repository for complete details.
 
-from __future__ import absolute_import, division, print_function
+
+import pickle
+
+import pytest
 
 from structlog._config import _CONFIG
 from structlog._generic import BoundLogger
-from structlog._loggers import ReturnLogger
+from structlog.testing import ReturnLogger
 
 
-class TestLogger(object):
+class TestLogger:
     def log(self, msg):
         return "log", msg
 
@@ -17,7 +20,7 @@ class TestLogger(object):
         return "gol", msg
 
 
-class TestGenericBoundLogger(object):
+class TestGenericBoundLogger:
     def test_caches(self):
         """
         __getattr__() gets called only once per logger method.
@@ -47,3 +50,30 @@ class TestGenericBoundLogger(object):
 
         assert "log", "foo" == b.log("foo")
         assert "gol", "bar" == b.gol("bar")
+
+    @pytest.mark.parametrize("proto", range(pickle.HIGHEST_PROTOCOL))
+    def test_pickle(self, proto):
+        """
+        Can be pickled and unpickled.
+
+        Works only on Python 3: TypeError: can't pickle instancemethod objects
+        """
+        b = BoundLogger(
+            ReturnLogger(),
+            _CONFIG.default_processors,
+            _CONFIG.default_context_class(),
+        ).bind(x=1)
+
+        assert b.info("hi") == pickle.loads(pickle.dumps(b, proto)).info("hi")
+
+    def test_deepcopy(self):
+        """
+        __getattr__ returns None for '__deepcopy__'
+        """
+        b = BoundLogger(
+            ReturnLogger(),
+            _CONFIG.default_processors,
+            _CONFIG.default_context_class(),
+        )
+
+        assert b.__deepcopy__ is None
