@@ -2,25 +2,29 @@
 # 2.0, and the MIT License.  See the LICENSE file in the root of this
 # repository for complete details.
 
-from __future__ import absolute_import, division, print_function
-
-import collections
+import json
 import logging
 import logging.config
 import os
+import sys
 
 import pytest
 
 from pretend import call_recorder
 
-from structlog import ReturnLogger, configure, get_logger, reset_defaults
+from structlog import (
+    PrintLogger,
+    ReturnLogger,
+    configure,
+    get_context,
+    reset_defaults,
+)
+from structlog._log_levels import _NAME_TO_LEVEL, CRITICAL, WARN
 from structlog.dev import ConsoleRenderer
 from structlog.exceptions import DropEvent
 from structlog.processors import JSONRenderer
 from structlog.stdlib import (
-    _NAME_TO_LEVEL,
-    CRITICAL,
-    WARN,
+    AsyncBoundLogger,
     BoundLogger,
     LoggerFactory,
     PositionalArgumentsFormatter,
@@ -30,11 +34,12 @@ from structlog.stdlib import (
     add_log_level_number,
     add_logger_name,
     filter_by_level,
+    get_logger,
     render_to_log_kwargs,
 )
+from structlog.types import BindableLogger
 
 from .additional_frame import additional_frame
-from .utils import py3_only
 
 
 def build_bl(logger=None, processors=None, context=None):
@@ -51,7 +56,7 @@ def return_method_name(_, method_name, __):
     return method_name
 
 
-class TestLoggerFactory(object):
+class TestLoggerFactory:
     def setup_method(self, method):
         """
         The stdlib logger factory modifies global state to fix caller
@@ -93,14 +98,15 @@ class TestLoggerFactory(object):
         assert file_name == os.path.realpath(__file__)
         assert func_name == "test_deduces_correct_caller"
 
-    @py3_only
     def test_stack_info(self):
+        """
+        If we ask for stack_info, it will returned.
+        """
         logger = _FixedFindCallerLogger("test")
         testing, is_, fun, stack_info = logger.findCaller(stack_info=True)
 
         assert "testing, is_, fun" in stack_info
 
-    @py3_only
     def test_no_stack_info_by_default(self):
         logger = _FixedFindCallerLogger("test")
         testing, is_, fun, stack_info = logger.findCaller()
@@ -119,8 +125,9 @@ class TestLoggerFactory(object):
         assert log_record.filename == os.path.basename(__file__)
 
     def test_sets_correct_logger(self):
-        assert logging.getLoggerClass() is logging.Logger
-
+        """
+        Calling LoggerFactory ensures that Logger.findCaller gets patched.
+        """
         LoggerFactory()
 
         assert logging.getLoggerClass() is _FixedFindCallerLogger
@@ -135,14 +142,21 @@ class TestLoggerFactory(object):
         assert "foo" == lf.name
 
 
-class TestFilterByLevel(object):
+class TestFilterByLevel:
     def test_filters_lower_levels(self):
+        """
+        Log entries below the current level raise a DropEvent.
+        """
         logger = logging.Logger(__name__)
         logger.setLevel(CRITICAL)
+
         with pytest.raises(DropEvent):
             filter_by_level(logger, "warn", {})
 
     def test_passes_higher_levels(self):
+        """
+        Log entries with higher levels are passed through unchanged.
+        """
         logger = logging.Logger(__name__)
         logger.setLevel(WARN)
         event_dict = {"event": "test"}
@@ -152,7 +166,7 @@ class TestFilterByLevel(object):
         assert event_dict is filter_by_level(logger, "exception", event_dict)
 
 
-class TestBoundLogger(object):
+class TestBoundLogger:
     @pytest.mark.parametrize(
         ("method_name"), ["debug", "info", "warning", "error", "critical"]
     )
@@ -191,6 +205,21 @@ class TestBoundLogger(object):
         assert "baz" == kwargs.get("bar")
         assert ("foo",) == kwargs.get("positional_args")
 
+    @pytest.mark.parametrize(
+        "attribute_name",
+        ["name", "level", "parent", "propagate", "handlers", "disabled"],
+    )
+    def test_stdlib_passthrough_attributes(self, attribute_name):
+        """
+        stdlib logger attributes are also available in stdlib BoundLogger.
+        """
+        stdlib_logger = logging.getLogger("Test")
+        stdlib_logger_attribute = getattr(stdlib_logger, attribute_name)
+        bl = BoundLogger(stdlib_logger, [], {})
+        bound_logger_attribute = getattr(bl, attribute_name)
+
+        assert bound_logger_attribute == stdlib_logger_attribute
+
     @pytest.mark.parametrize(
         "method_name,method_args",
         [
@@ -263,8 +292,40 @@ class TestBoundLogger(object):
             "event", exc_info=42
         )
 
+    def test_proxies_bind(self):
+        """
+        Bind calls the correct bind.
+        """
+        bl = build_bl().bind(a=42)
 
-class TestPositionalArgumentsFormatter(object):
+        assert {"a": 42} == get_context(bl)
+
+    def test_proxies_new(self):
+        """
+        Newcalls the correct new.
+        """
+        bl = build_bl().bind(a=42).new(b=23)
+
+        assert {"b": 23} == get_context(bl)
+
+    def test_proxies_unbind(self):
+        """
+        Unbind calls the correct unbind.
+        """
+        bl = build_bl().bind(a=42).unbind("a")
+
+        assert {} == get_context(bl)
+
+    def test_proxies_try_unbind(self):
+        """
+        try_unbind calls the correct try_unbind.
+        """
+        bl = build_bl().bind(a=42).try_unbind("a", "b")
+
+        assert {} == get_context(bl)
+
+
+class TestPositionalArgumentsFormatter:
     def test_formats_tuple(self):
         """
         Positional arguments as simple types are rendered.
@@ -329,7 +390,7 @@ class TestPositionalArgumentsFormatter(object):
         assert {} == formatter(None, None, {"positional_args": ()})
 
 
-class TestAddLogLevelNumber(object):
+class TestAddLogLevelNumber:
     @pytest.mark.parametrize("level, number", _NAME_TO_LEVEL.items())
     def test_log_level_number_added(self, level, number):
         """
@@ -340,7 +401,7 @@ class TestAddLogLevelNumber(object):
         assert number == event_dict["level_number"]
 
 
-class TestAddLogLevel(object):
+class TestAddLogLevel:
     def test_log_level_added(self):
         """
         The log level is added to the event dict.
@@ -380,7 +441,7 @@ def log_record():
     return create_log_record
 
 
-class TestAddLoggerName(object):
+class TestAddLoggerName:
     def test_logger_name_added(self):
         """
         The logger name is added to the event dict.
@@ -402,7 +463,7 @@ class TestAddLoggerName(object):
         assert name == event_dict["logger"]
 
 
-class TestRenderToLogKW(object):
+class TestRenderToLogKW:
     def test_default(self):
         """
         Translates `event` to `msg` and handles otherwise empty `event_dict`s.
@@ -440,7 +501,7 @@ def configure_for_pf():
     reset_defaults()
 
 
-def configure_logging(pre_chain):
+def configure_logging(pre_chain, logger=None, pass_foreign_args=False):
     """
     Configure logging to use ProcessorFormatter.
     """
@@ -454,6 +515,8 @@ def configure_logging(pre_chain):
                     "processor": ConsoleRenderer(colors=False),
                     "foreign_pre_chain": pre_chain,
                     "format": "%(message)s [in %(funcName)s]",
+                    "logger": logger,
+                    "pass_foreign_args": pass_foreign_args,
                 }
             },
             "handlers": {
@@ -474,7 +537,7 @@ def configure_logging(pre_chain):
     )
 
 
-class TestProcessorFormatter(object):
+class TestProcessorFormatter:
     """
     These are all integration tests because they're all about integration.
     """
@@ -485,11 +548,6 @@ class TestProcessorFormatter(object):
         to logging.
         """
         configure_logging(None)
-        configure(
-            processors=[ProcessorFormatter.wrap_for_formatter],
-            logger_factory=LoggerFactory(),
-            wrapper_class=BoundLogger,
-        )
 
         logging.getLogger().warning("foo")
 
@@ -511,6 +569,24 @@ class TestProcessorFormatter(object):
             "hello world. [in test_clears_args]\n",
         ) == capsys.readouterr()
 
+    def test_pass_foreign_args_true_sets_positional_args_key(
+        self, configure_for_pf, capsys
+    ):
+        """
+        If `pass_foreign_args` is `True` we set the `positional_args` key in
+        the `event_dict` before clearing args.
+        """
+        test_processor = call_recorder(lambda l, m, event_dict: event_dict)
+        configure_logging((test_processor,), pass_foreign_args=True)
+
+        positional_args = {"foo": "bar"}
+        logging.getLogger().info("okay %(foo)s", positional_args)
+
+        event_dict = test_processor.calls[0].args[2]
+
+        assert "positional_args" in event_dict
+        assert positional_args == event_dict["positional_args"]
+
     def test_log_dict(self, configure_for_pf, capsys):
         """
         Test that dicts can be logged with std library loggers.
@@ -530,11 +606,6 @@ class TestProcessorFormatter(object):
         non-structlog log entries.
         """
         configure_logging((add_log_level,))
-        configure(
-            processors=[ProcessorFormatter.wrap_for_formatter],
-            logger_factory=LoggerFactory(),
-            wrapper_class=BoundLogger,
-        )
 
         logging.getLogger().warning("foo")
 
@@ -548,11 +619,6 @@ class TestProcessorFormatter(object):
         foreign_pre_chain works with add_logger_name processor.
         """
         configure_logging((add_logger_name,))
-        configure(
-            processors=[ProcessorFormatter.wrap_for_formatter],
-            logger_factory=LoggerFactory(),
-            wrapper_class=BoundLogger,
-        )
 
         logging.getLogger("sample-name").warning("foo")
 
@@ -591,11 +657,6 @@ class TestProcessorFormatter(object):
         """
         test_processor = call_recorder(lambda l, m, event_dict: event_dict)
         configure_logging((test_processor,))
-        configure(
-            processors=[ProcessorFormatter.wrap_for_formatter],
-            logger_factory=LoggerFactory(),
-            wrapper_class=BoundLogger,
-        )
 
         try:
             raise RuntimeError("oh noo")
@@ -607,6 +668,31 @@ class TestProcessorFormatter(object):
         assert "exc_info" in event_dict
         assert isinstance(event_dict["exc_info"], tuple)
 
+    def test_foreign_pre_chain_sys_exc_info(self, configure_for_pf, capsys):
+        """
+        If a foreign_pre_chain function accesses sys.exc_info(),
+        ProcessorFormatter should not have changed it.
+        """
+
+        class MyException(Exception):
+            pass
+
+        def add_excinfo(logger, log_method, event_dict):
+            event_dict["exc_info"] = sys.exc_info()
+            return event_dict
+
+        test_processor = call_recorder(lambda l, m, event_dict: event_dict)
+        configure_logging((add_excinfo, test_processor))
+
+        try:
+            raise MyException("oh noo")
+        except Exception:
+            logging.getLogger().error("okay")
+
+        event_dict = test_processor.calls[0].args[2]
+
+        assert MyException is event_dict["exc_info"][0]
+
     def test_other_handlers_get_original_record(
         self, configure_for_pf, capsys
     ):
@@ -643,7 +729,6 @@ class TestProcessorFormatter(object):
         logger = logging.getLogger()
 
         def format_exc_info_fake(logger, name, event_dict):
-            event_dict = collections.OrderedDict(event_dict)
             del event_dict["exc_info"]
             event_dict["exception"] = "Exception!"
             return event_dict
@@ -673,7 +758,6 @@ class TestProcessorFormatter(object):
             assert "Traceback (most recent call last):" in err
 
     @pytest.mark.parametrize("keep", [True, False])
-    @py3_only
     def test_formatter_unsets_stack_info(self, configure_for_pf, capsys, keep):
         """
         Stack traces doesn't get printed outside of the json document when
@@ -713,3 +797,169 @@ class TestProcessorFormatter(object):
             "",
             "[warning  ] foo [in test_native]\n",
         ) == capsys.readouterr()
+
+    def test_native_logger(self, configure_for_pf, capsys):
+        """
+        If the log entry comes from structlog, it's unpackaged and processed.
+        """
+        logger = logging.getLogger()
+        configure_logging(None, logger=logger)
+
+        get_logger().warning("foo")
+
+        assert (
+            "",
+            "[warning  ] foo [in test_native_logger]\n",
+        ) == capsys.readouterr()
+
+    def test_foreign_pre_chain_filter_by_level(self, configure_for_pf, capsys):
+        """
+        foreign_pre_chain works with filter_by_level processor.
+        """
+        logger = logging.getLogger()
+        configure_logging((filter_by_level,), logger=logger)
+        configure(
+            processors=[ProcessorFormatter.wrap_for_formatter],
+            logger_factory=LoggerFactory(),
+            wrapper_class=BoundLogger,
+        )
+
+        logger.warning("foo")
+
+        assert (
+            "",
+            "foo [in test_foreign_pre_chain_filter_by_level]\n",
+        ) == capsys.readouterr()
+
+
+@pytest.fixture(name="abl")
+async def _abl(cl):
+    return AsyncBoundLogger(cl, context={}, processors=[])
+
+
+@pytest.mark.skipif(
+    sys.version_info[:2] < (3, 7),
+    reason="AsyncBoundLogger is only for Python 3.7 and later.",
+)
+class TestAsyncBoundLogger:
+    @pytest.mark.asyncio
+    async def test_protocol(self, abl):
+        """
+        AsyncBoundLogger is a proper BindableLogger.
+        """
+        assert isinstance(abl, BindableLogger)
+
+    @pytest.mark.asyncio
+    async def test_correct_levels(self, abl, cl, stdlib_log_method):
+        """
+        The proxy methods call the correct upstream methods.
+        """
+        await getattr(abl.bind(foo="bar"), stdlib_log_method)("42")
+
+        aliases = {"exception": "error", "warn": "warning"}
+
+        alias = aliases.get(stdlib_log_method)
+        if alias:
+            expect = alias
+        else:
+            expect = stdlib_log_method
+
+        assert expect == cl.calls[0].method_name
+
+    @pytest.mark.asyncio
+    async def test_log_method(self, abl, cl):
+        """
+        The `log` method is proxied too.
+        """
+        await abl.bind(foo="bar").log(logging.ERROR, "42")
+
+        assert "error" == cl.calls[0].method_name
+
+    @pytest.mark.asyncio
+    async def test_exception(self, abl, cl):
+        """
+        `exception` makes sure 'exc_info" is set, if it's not set already.
+        """
+        try:
+            raise ValueError("omg")
+        except ValueError:
+            await abl.exception("oops")
+
+        ei = cl.calls[0].kwargs["exc_info"]
+
+        assert ValueError is ei[0]
+        assert ("omg",) == ei[1].args
+
+    @pytest.mark.asyncio
+    async def test_exception_do_not_overwrite(self, abl, cl):
+        """
+        `exception` leaves exc_info be, if it's set.
+        """
+        o1 = object()
+        o2 = object()
+        o3 = object()
+
+        try:
+            raise ValueError("omg")
+        except ValueError:
+            await abl.exception("oops", exc_info=(o1, o2, o3))
+
+        ei = cl.calls[0].kwargs["exc_info"]
+        assert (o1, o2, o3) == ei
+
+    @pytest.mark.asyncio
+    async def test_bind_unbind(self, cl):
+        """
+        new/bind/unbind/try_unbind are correctly propagated.
+        """
+        l1 = AsyncBoundLogger(cl, context={}, processors=[])
+
+        l2 = l1.bind(x=42)
+
+        assert l1 is not l2
+        assert l1.sync_bl is not l2.sync_bl
+        assert {} == l1._context
+        assert {"x": 42} == l2._context
+
+        l3 = l2.new(y=23)
+
+        assert l2 is not l3
+        assert l2.sync_bl is not l3.sync_bl
+        assert {"y": 23} == l3._context
+
+        l4 = l3.unbind("y")
+
+        assert {} == l4._context
+        assert l3 is not l4
+
+        # N.B. x isn't bound anymore.
+        l5 = l4.try_unbind("x")
+
+        assert {} == l5._context
+        assert l4 is not l5
+
+    @pytest.mark.asyncio
+    async def test_integration(self, capsys):
+        """
+        Configure and log an actual entry.
+        """
+
+        configure(
+            processors=[add_log_level, JSONRenderer()],
+            logger_factory=PrintLogger,
+            wrapper_class=AsyncBoundLogger,
+            cache_logger_on_first_use=True,
+        )
+
+        logger = get_logger()
+
+        await logger.bind(foo="bar").info("baz", x="42")
+
+        assert {
+            "foo": "bar",
+            "x": "42",
+            "event": "baz",
+            "level": "info",
+        } == json.loads(capsys.readouterr().out)
+
+        reset_defaults()
