@@ -2,18 +2,26 @@
 # 2.0, and the MIT License.  See the LICENSE file in the root of this
 # repository for complete details.
 
-from __future__ import absolute_import, division, print_function
 
 import threading
 
-from collections import OrderedDict
-
 import pytest
 
 from structlog._base import BoundLoggerBase
 from structlog._config import wrap_logger
-from structlog._loggers import ReturnLogger
-from structlog.threadlocal import as_immutable, tmp_bind, wrap_dict
+from structlog.testing import ReturnLogger
+from structlog.threadlocal import (
+    _CONTEXT,
+    _get_context,
+    as_immutable,
+    bind_threadlocal,
+    clear_threadlocal,
+    merge_threadlocal,
+    merge_threadlocal_context,
+    tmp_bind,
+    unbind_threadlocal,
+    wrap_dict,
+)
 
 
 try:
@@ -33,9 +41,9 @@ def D():
 @pytest.fixture
 def log(logger):
     """
-    Returns a ReturnLogger with a freshly wrapped OrderedDict.
+    Returns a ReturnLogger with a freshly wrapped dict.
     """
-    return wrap_logger(logger, context_class=wrap_dict(OrderedDict))
+    return wrap_logger(logger, context_class=wrap_dict(dict))
 
 
 @pytest.fixture
@@ -47,7 +55,7 @@ def logger():
     return ReturnLogger()
 
 
-class TestTmpBind(object):
+class TestTmpBind:
     def test_bind(self, log):
         """
         tmp_bind does not modify the thread-local state.
@@ -78,7 +86,7 @@ class TestTmpBind(object):
         assert {"y": 23} == log._context._dict
 
 
-class TestAsImmutable(object):
+class TestAsImmutable:
     def test_does_not_affect_global(self, log):
         """
         A logger from as_mutable is independent from thread local state.
@@ -112,7 +120,7 @@ class TestAsImmutable(object):
         assert isinstance(as_immutable(il), BoundLoggerBase)
 
 
-class TestThreadLocalDict(object):
+class TestThreadLocalDict:
     def test_wrap_returns_distinct_classes(self):
         """
         Each call to wrap_dict returns a distinct new class whose context is
@@ -262,3 +270,85 @@ class TestThreadLocalDict(object):
         The context of a new wrapped class is empty.
         """
         assert 0 == len(D())
+
+
+class TestNewThreadLocal:
+    def test_alias(self):
+        """
+        We're keeping the old alias around.
+        """
+        assert merge_threadlocal_context is merge_threadlocal
+
+    def test_bind_and_merge(self):
+        """
+        Binding a variable causes it to be included in the result of
+        merge_threadlocal.
+        """
+        clear_threadlocal()
+        bind_threadlocal(a=1)
+
+        assert {"a": 1, "b": 2} == merge_threadlocal(None, None, {"b": 2})
+
+    def test_clear(self):
+        """
+        The thread-local context can be cleared, causing any previously bound
+        variables to not be included in merge_threadlocal's result.
+        """
+        bind_threadlocal(a=1)
+        clear_threadlocal()
+
+        assert {"b": 2} == merge_threadlocal(None, None, {"b": 2})
+
+    def test_merge_works_without_bind(self):
+        """
+        merge_threadlocal returns values as normal even when there has
+        been no previous calls to bind_threadlocal.
+        """
+        clear_threadlocal()
+
+        assert {"b": 2} == merge_threadlocal(None, None, {"b": 2})
+
+    def test_multiple_binds(self):
+        """
+        Multiple calls to bind_threadlocal accumulate values instead of
+        replacing them.
+        """
+        clear_threadlocal()
+        bind_threadlocal(a=1, b=2)
+        bind_threadlocal(c=3)
+
+        assert {"a": 1, "b": 2, "c": 3} == merge_threadlocal(
+            None, None, {"b": 2}
+        )
+
+    def test_unbind_threadlocal(self):
+        """
+        Test that unbinding from threadlocal works for keys that exist
+        and does not raise error when they do not exist.
+        """
+
+        clear_threadlocal()
+        bind_threadlocal(a=234, b=34)
+
+        assert {"a": 234, "b": 34} == merge_threadlocal_context(None, None, {})
+
+        unbind_threadlocal("a")
+
+        assert {"b": 34} == merge_threadlocal_context(None, None, {})
+
+        unbind_threadlocal("non-existing-key")
+
+        assert {"b": 34} == merge_threadlocal_context(None, None, {})
+
+    def test_get_context_no_context(self):
+        """
+        If there is no context yet, _get_context will add it.
+        """
+        # Don't rely on test order.
+        if hasattr(_CONTEXT, "context"):
+            del _CONTEXT.context
+
+        with pytest.raises(AttributeError):
+            _CONTEXT.context
+
+        assert {} == _get_context()
